/* Model kuvaa tietokannan taulujen rakennetta
https://sequelize.org/master/class/lib/model.js~Model.html
Kannan molemmille tauluille on luokka jossa on taulun kuvaus

Modeliin voi asettaa monenlaisia rajoittimia ja validointeja
https://sequelize.org/master/manual/validations-and-constraints.html
*/

const Sequelize = require('sequelize');
const Model = Sequelize.Model;
const conn = require('./dbconnection');

//Student
class Student extends Model {}
Student.init(
  {
    studentid: {
      type: Sequelize.INTEGER,
      allowNull: false,
      primaryKey: true,
      autoIncrement: true,
      validate: {
        len: [1, 10], // pituuden validointi
      },
    },
    studentcode: {
      type: Sequelize.STRING,
      allowNull: false,
      validate: {
        is: /[a-z]{1}[0-9]{4}/, // opiskelijanumeron validointi regex:llä
      },
    },
    name: {
      type: Sequelize.STRING,
    },
    email: {
      type: Sequelize.STRING,
      validate: {
        isEmail: true, // emailin muodon validointi
      },
    },
    studypoints: {
      type: Sequelize.INTEGER,
    },
  },
  { sequelize: conn, modelName: 'Student' }
);

//Grade
class Grade extends Model {}
Grade.init(
  {
    gradeid: {
      type: Sequelize.INTEGER,
      allowNull: false,
      primaryKey: true,
      autoIncrement: true,
    },
    studentcode: {
      type: Sequelize.STRING,
      allowNull: false,
      validate: {
        is: /[a-z]{1}[0-9]{4}/,
      }
    },
    coursecode: {
      type: Sequelize.STRING,
    },
    grade: {
      type: Sequelize.INTEGER,
    }
  },
  { sequelize: conn, modelName: 'Grade' }
);

// Yhdistetään Student ja Grade. Molemmissa on studentcode -kenttä
Student.hasMany(Grade, { foreignKey: 'studentcode', sourceKey: 'studentcode' });
Grade.belongsTo(Student, {
  foreignKey: 'studentcode',
  targetKey: 'studentcode',
});

module.exports = { Student, Grade };
